"""
	This file handles customer & plans associated with a given product/app
	The subscription works as follows:
	- 
"""
from __future__ import division
from flask import Flask, request, session, render_template,Response
from flask_cors import CORS
import Domain
from couchdbkit import Server, Document
import stripe
import json
from StringIO import StringIO
import re
import os
from utils.params import PARAMS
from utils.transport import Couchdb, CouchdbReader
PORT = 8100 if 'port' not in PARAMS else int(PARAMS['port']) ;
path = PARAMS['path'] #os.environ['CONFIG']
f = open(path)
CONFIG = json.loads(f.read())
CONTEXT= PARAMS['context'] if 'context' in PARAMS else ''
stripe_keys = {
    'secret_key': CONFIG['stripe']['secret'].strip(),
    'publishable_key': CONFIG['stripe']['pub'].strip()
}

stripe.api_key = stripe_keys['secret_key'].strip()


	
app = Flask(__name__)
CORS(app)
COUCHDB = Server(uri=CONFIG['couchdb']['uri']) ;
"""
	This function will set the user information to the session and update the information
	@header uid	user email address
"""
@app.route("/init/<app_name>",methods=['POST'])
def init(app_name):
	plans = []
	handler = None
	uid = request.headers['uid']
	plan = request.headers['pid']
	auid = None
	if 'uid' in request.headers and 'auid' not in request.headers:
		id = uid
		#couchdb = Couchdb(uri=CONFIG['couchdb']['uri'],dbname=app_name,uid=uid,create=False)
		#DB = couchdb.dbase
	if 'auid' in request.headers :
		auid = request.headers['auid']
		auid = json.loads(auid)if auid.startswith('[') else auid 
		if isinstance(auid,list) :
			auid = list(set(auid) - set([uid]))

			if len(auid) == 0:
				auid = None
				id = uid
			else:
				auid = auid[0]
				id = auid
	couchdb = Couchdb(uri=CONFIG['couchdb']['uri'],dbname=app_name,uid=id,create=False)
	DB = couchdb.dbase
		#handler.update_user(id=handler.getId(auid),uid=uid)
	
	handler = Domain.User(DB,stripe) ;
	if auid is None :
		handler.init(uid,plan) ;
	else:
		if couchdb.isready() == False:
			handler.init(auid,plan)
			id = handler.getId(uid) 

		handler.initialize(auid)
		id = handler.getId(auid)
		handler.update_user(id=id,uid=uid) ;
	
	plans = handler.plans()
	#return json.dumps(plans)
	_features = features(app_name)
	return _features
"""
	This function will update the user's email
"""
@app.route('/init/<app_name>',methods=['PUT'])	
def init_update(app_name):
	uid	= request.headers['uid']
	id 	= session['customer.id']
	couchdb = Couchdb(uri=CONFIG['couchdb']['uri'],dbname=app_name,uid=None)
	DB	= couchdb.dbase
	handler = Domain.User(DB,stripe)
	handler.update_user(uid=uid,id=id)
	
	return ('',204)
	

"""
	This function subscribes a user to a given service for an application
	This function guarantees not to duplicate subscriptions
	@resource 	name	name of the application {cloud-music}
	@header		key	service/plan
"""
@app.route('/subscribe/<app_name>',methods=['POST'])
def subscribe(app_name):
	#
	# The name is the full name of the service
	#
	resp = "0"
	if 'stripeToken' in request.form :
		stripeToken	= request.form['stripeToken']
		uid		= request.form['stripeEmail']
		tokenType	= request.form['stripeTokenType']
		amount		= request.form['amount']
		pid 		= request.form['plan']

	else:
		pid = request.headers['pid']
		uid = request.headers['uid']
		stripeToken = None
	
	plans = stripe.Plan.list().data	
	plan = [item for item in plans if item.id == pid]
	resp = "0"

	couch_handler = Couchdb(uri=CONFIG['couchdb']['uri'],dbname=app_name,uid=uid)
	DB = couch_handler.dbase
	handler = Domain.User(DB,stripe,stripeToken) ;
	
	if plan :
		handler.init(uid,plan)
	resp = plan[0].id

	return ('',204)

"""
	This function returns the features of a user for a given application
	The features are returned provided the plan is still valid i.e (end_date,status={active|trailing})
	@header uid	user's email
"""
@app.route('/features/<app_name>')
def features(app_name):
	plans = []
	if 'uid' in request.headers :
		uid = request.headers['uid']
		
		couchdb = CouchdbReader(uri=CONFIG['couchdb']['uri'],dbname=app_name,uid=uid,create=False)
		handler = Domain.User(couchdb.dbase,stripe)
		handler.initialize(uid)
		lsub = handler.subscriptions()
		
		plans = [ sub['plan'] for sub in lsub if sub['ended_at'] is None  ]
		plans = [item['metadata'] for item in plans if item['id']]

	return json.dumps(plans)
"""
	This function returns a user's plans/status for an application
	@header uid 	user's email address
"""
@app.route('/status/<app_name>')
def status(app_name):
	
	uid = request.headers['uid']
	
	couchdb = CouchdbReader(uri=CONFIG['couchdb']['uri'],dbname=app_name,uid=uid,create=False)
	handler = Domain.User(couchdb.dbase,stripe)
	handler.initialize(uid)
	#lsub = handler.subscriptions()
	plans = handler.plans()
	return json.dumps(plans)
"""
	This endpoint is the signup form for a given application,
	It will allow user's to be able to signup for various plans
	@pre 'uid' in request.headers
"""	
@app.route('/signup/<app_name>')
def get_plans(app_name) :
	apikey = CONFIG['stripe']['pub'].strip()	
	#
	# This function returns the plans for a given application
	# We assume the application name is the prefix of the plan identifier in stripe
	#
	uid = session['uid'] if 'uid' in session else None
	can_purchase = True
	if uid is None :
		uid = request.args.get('uid') if 'uid' in request.args else None
	plans = stripe.Plan.list(limit=10)
	
	# plans = handler.cast(plans.data)
	plans = json.loads(json.dumps(plans.data, sort_keys=True, indent=2))
	
	plans = [item for item in plans if re.match(app_name,item['name'])]
	if uid is not None:
		couchdb = Couchdb(uri=CONFIG['couchdb']['uri'],dbname=app_name,uid=uid,create=False)
		handler = Domain.User(couchdb.dbase,stripe)
		if handler.getId(uid) is not None:
			handler.init(uid,[])
			subs = handler.subscriptions().data
			ids = [item.id for item in subs]
			for item in plans :
				if item['id'] in ids :
					item['active'] = True
			#
			# let's determine if this user can make a purchase
			#
			
	else:
		#			#
		# This should address any case of the database of customers being out of sync
		# @TODO: Have a log suggesting a utility imports the customer
		#
		return json.dumps(plans)

	#
	# @TODO: Mark the plans the current user is signed up for
	#
	return render_template('subscribe.html',context=CONTEXT,uid=uid,app_name=app_name,plans=plans,apikey=apikey)
	
@app.route('/subscribe/<name>',methods=['DELETE'])
def cancel_subscribe(name) :
	pass
"""
	This function defines if a given user is a customer or not
	We should be able to tell by how we create customers
"""
@app.route('/checkout/<app_name>',methods=['GET'])
def is_customer (app_name):
	uid = request.args.get('uid')
	pid = request.args.get('pid')
	couchdb = CouchdbReader(uri=CONFIG['couchdb']['uri'],dbname=app_name,uid=uid,create=False)
	r = couchdb.view('federation/uid_map',key=uid)
	id = r[0]['value']
	couchdb = CouchdbReader(uri=CONFIG['couchdb']['uri'],dbname=app_name,uid=id,create=False)
	info = couchdb.read()
	lsub = info['subscriptions']
	
	plans = [dict(dict(item['plan'],**{"status":item['status']}),**{"subscription":item['id']}) for item in lsub]
	if pid is not None:
		plans = [item for item in plans if item['id'] == pid]
	#
	# Caching the subscription identifiers so we can create an invoice later on (if need be)
	# @TODO Improve this process later on by allowing user's to pay for what they can (not everything)
	#
	session['plans'] = plans

	session['plans']
	amount = sum([item['amount'] for item in plans])	
	apikey = CONFIG['stripe']['pub'].strip()
	
	amount = amount / 100
	return render_template('bill.html',context=CONTEXT,apikey=apikey,app_name=app_name.replace('-',' '),plans=plans,total_amount=amount)

if __name__ == '__main__' :
	app.debug = True ;
	app.secret_key = '360-8y-[0v@t10n]+kr81v17y'
	app.run(port=PORT,threaded=True)
